////////////////////////////////////////////////////////////////////////////
//  
//  SymLinkProps - Link properties plugin for the Konqueror file manager
//  
//  Copyright (c) 2004 Jonathan Marten <jjm@keelhaul.demon.co.uk>
//  Home and download page:  http://www.keelhaul.demon.co.uk/kde/
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  It is distributed in the hope that it will be useful, but WITHOUT ANY
//  WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; see the file COPYING for further details.
//  If not, write to the Free Software Foundation, Inc., 59 Temple Place -
//  Suite 330, Boston, MA 02111-1307, USA.
//
////////////////////////////////////////////////////////////////////////////

#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qapplication.h>

#include <kurl.h>
#include <kfileitem.h>
#include <kmessagebox.h>
#include <kgenericfactory.h>
#include <kfiledialog.h>

#include <kaboutdata.h>
#include <khelpmenu.h>
#include <kapplication.h>
#include <kdebug.h>

#include <kio/job.h>

#include <unistd.h>

#include "config.h"
#include "symlinkprops.h"


//  Data for displaying the "About" dialogue

static const KAboutData aboutdata(PACKAGE,
				  I18N_NOOP("SymLinkProps Plugin"),
				  VERSION,
				  I18N_NOOP("Symbolic link properties plugin"),
				  KAboutData::License_GPL,
				  "(c) 2004 Jonathan Marten",
				  NULL,
				  "http://www.keelhaul.demon.co.uk",
				  "jjm@keelhaul.demon.co.uk");


//  Constructor.  Look at the selected item, and add the link properties page
//  to the parent dialogue if applicable.

SymlinkPropsDialogPlugin::SymlinkPropsDialogPlugin(KPropertiesDialog *dialog,
						   const char *,
						   const QStringList &)
	: KPropsDlgPlugin(dialog)
{
	helpmenu = NULL;

	url = dialog->kurl();

	KFileItem fi(KFileItem::Unknown,KFileItem::Unknown,url,true);
	kdDebug(0) << k_funcinfo "url=" << url.url() << " islink=" << fi.isLink() << " dest=" << fi.linkDest() << endl;
	if (!fi.isLink()) return;

	widget = dialog->addPage(i18n("S&ymbolic Link"));

	QGridLayout *layout = new QGridLayout(widget,4,3,
					      KDialog::marginHint(),
					      KDialog::spacingHint());
	layout->setColStretch(1,1);
	layout->setRowStretch(1,1);

	bool changeable = true;
	if (fi.isLocalFile())
	{
		QString parent = url.directory(true,true);
//		kdDebug(0) << k_funcinfo << "local, parent=" << parent.latin1() << endl;
		if (access(parent.latin1(),W_OK)!=0) changeable = false;
	}

	dest = new QLineEdit(widget);
	dest->setText(fi.linkDest());
	dest->setReadOnly(!changeable);
	connect(dest,SIGNAL(textChanged(const QString &)),this,SIGNAL(changed()));
	layout->addWidget(dest,0,1);

	QLabel *label = new QLabel(dest,i18n("&Link to:"),widget);
	layout->addWidget(label,0,0);

	QPushButton *browse = new QPushButton(i18n("&Browse..."),widget);
	browse->setEnabled(changeable);
	connect(browse,SIGNAL(clicked()),this,SLOT(slotBrowseDest()));
	layout->addWidget(browse,0,2);

	QPushButton *pbabout = new QPushButton(i18n("&About..."),widget);
	connect(pbabout,SIGNAL(clicked()),this,SLOT(slotAbout()));
	layout->addWidget(pbabout,2,2);

	QPushButton *pbhelp = new QPushButton(i18n("&Help"),widget);
	connect(pbhelp,SIGNAL(clicked()),this,SLOT(slotHelp()));
	layout->addWidget(pbhelp,3,2);

}


//  Destructor.  Deallocate the help menu if it was allocated.

SymlinkPropsDialogPlugin::~SymlinkPropsDialogPlugin()
{
	if (helpmenu!=NULL) delete helpmenu;
}


//  The "Browse" button was clicked.  Display a file dialogue to select a
//  new destination for the link.

void SymlinkPropsDialogPlugin::slotBrowseDest()
{
	if (dest==NULL || dest->isReadOnly()) return;

	QString p = url.upURL().url(1);
//	kdDebug(0) << k_funcinfo << "url='" << url << "' up='" << p << "'" << endl;

	KFileDialog *dlg = new KFileDialog(p,QString::null,widget,"",true);
	dlg->setOperationMode(KFileDialog::Other);
	dlg->setMode(KFile::LocalOnly|KFile::File|KFile::Directory);
	dlg->setLocationLabel(i18n("Link to:"));

	if (!dlg->exec())
	{
		delete dlg;
		return;
	}

	KURL f = dlg->selectedURL();
	delete dlg;
//	kdDebug(0) << k_funcinfo << "f='" << f.url() << "'" << endl;

	if (f.isEmpty()) return;
	if (!f.isLocalFile())				// shouldn't happen,
	{						// dialogue enforces
		KMessageBox::sorry(widget,i18n("A symbolic link can only be made to a local destination"));
		return;
	}

	dest->setText(f.path());
}


//  The "OK" button in the parent dialogue was clicked.  If a new
//  destination has been entered, implement the change.
//
//  Using of the two functions below to wait for the job to finish is
//  copied from kdelibs/kio/kfile/kpropertiesdialog.cpp, which also
//  says "Don't do this at home" :-)

extern void qt_enter_modal(QWidget *widget);
extern void qt_leave_modal(QWidget *widget);

void SymlinkPropsDialogPlugin::applyChanges()
{
	if (dest==NULL || dest->isReadOnly()) return;

	QString path = url.path();
	QString newdest = dest->text();
//	kdDebug(0) << k_funcinfo << "path='" << path << "'" << " newdest='" << newdest << "'" << endl;
	if (newdest.isEmpty())
	{
		KMessageBox::sorry(widget,i18n("Link destination must be entered"));
		return;
	}

	KIO::Job *job = KIO::symlink(newdest,path,true,false);
	connect(job,SIGNAL(result(KIO::Job *)),SLOT(slotSymlinkResult(KIO::Job *)));

	// Wait for job
	QWidget dummy(0,0,WType_Dialog|WShowModal);
	qt_enter_modal(&dummy);
	qApp->enter_loop();
	qt_leave_modal(&dummy);
}


//  The KIO job has finished, hopefully successfully.

void SymlinkPropsDialogPlugin::slotSymlinkResult(KIO::Job *job)
{
	if (job->error()) job->showErrorDialog(widget);
	qApp->exit_loop();
}


//  Integration into Konqueror's properties dialogue.

typedef KGenericFactory<SymlinkPropsDialogPlugin,KPropertiesDialog> SymlinkPropsDialogPluginFactory;

K_EXPORT_COMPONENT_FACTORY(symlinkprops,
			   SymlinkPropsDialogPluginFactory("symlink"))


//  Display the plugin information.

void SymlinkPropsDialogPlugin::slotAbout()
{
	if (helpmenu==NULL) helpmenu = new KHelpMenu(widget,&aboutdata);
	helpmenu->aboutApplication();
}


//  Open the KDE Help Centre at the plugin documentation.

void SymlinkPropsDialogPlugin::slotHelp()
{
	kapp->invokeHelp("",("konq-plugins/" PACKAGE));
}


#include "symlinkprops.moc"
